import time

cache_root = '/home/psha/tmp/mgp-cache/'

from session import Session, SSLData

class CryptoData(SSLData):
	def __init__(self, node):
		SSLData.__init__(self)
		ta = filter(lambda x: x.type == "attribute" and x.name == "encoding", node.properties)
		if len(ta) > 0:
			self.enc = ta[0].content
		if not self.enc in ["PEM"]:
			raise RuntimeError("Unable to proceed with unknown encoding: %s" % self.enc)
		self.type = node.name
		self.data = node.content

class CachedSession(Session):
	def __init__(self, session, load = False):
		Session.__init__(self, session)
		self.filename = "%s/%s.xml" % (cache_root, self.session)
		if load:
			self.files.append(self.filename)
			self.load()

	def load_elem(self, elem):
		if elem.name == "user": self.user = elem.content
		elif elem.name == "auth": self.auth = elem.content
		elif elem.name == "scheme": self.scheme = elem.content
		elif elem.name == "cert": self.cert = CryptoData(elem)
		elif elem.name == "key": self.key = CryptoData(elem)
		elif elem.name == "proxy": 
			self.proxy = elem.content
			self.files.append(self.proxy)
	def load_attr(self, attr):
		if attr.name == "name" and self.session != attr.content:
			raise RuntimeError("Session [%s] broken" % (self.session))
		elif attr.name == "expires":
			self.expires = int(attr.content)
	def load(self):
		import libxml2
		xf = libxml2.parseFile(self.filename)
		xr = xf.getRootElement()
		map( self.load_elem, filter(lambda x: x.type == 'element', xr.children))
		map( self.load_attr, filter(lambda x: x.type == 'attribute', xr.properties))
	def store(self):
		tmp = self.xml()
		f = file(self.filename, "w")
		f.write(tmp)
		f.close()
	def xml(self):
		extended = ""
		if self.key: extended = extended + "\t<mgp:key encoding='%s'>%s</mgp:key>\n" % (
				self.key.enc, self.key.data)
		if self.cert: extended = extended + "\t<mgp:cert encoding='%s'>%s</mgp:cert>\n" % (
				self.cert.enc, self.cert.data)
		if self.proxy: extended = extended + "\t<mgp:proxy>%s</mgp:proxy>\n" % (self.proxy)
		return """<?xml version='1.0' encoding='utf-8'?>

<mgp:session name='%s' expires='%d' xmlns:mgp='http://psha.org.ru/ns/mgp'>
	<mgp:user>%s</mgp:user>
	<mgp:auth>%s</mgp:auth>
	<mgp:scheme>%s</mgp:scheme>
%s</mgp:session>
""" % (self.session, self.expires, self.user, self.auth, self.scheme, extended)
